#include "Builder/Builders/CommonBuilder.h"
#include "MazeContent/Entities/DynamicEntities/Enemy.h"
#include "MazeContent/Entities/StaticEntities/Food.h"

CommonBuilder::CommonBuilder(float width, float height, float room_size):
                            m_width(width), m_height(height), m_room_size(room_size) {}

void CommonBuilder::create_context(float dynamic_objects_ratio) {
    std::vector<Room*> buffer;
    for (auto& row : m_rooms) {
        for (auto& room : row) {
            buffer.push_back(room.get());
        }
    }

    int pos = std::rand() % buffer.size();
    m_context.pacman.set_location(buffer.at(pos));
    buffer.erase(buffer.begin() + pos);

    size_t rooms_in_col = static_cast<size_t>(m_height / m_room_size) % 2 == 0 ? m_height / m_room_size - 1 : m_height / m_room_size;
    size_t rooms_in_row = static_cast<size_t>(m_width / m_room_size) % 2 == 0 ? m_width / m_room_size - 1 : m_width / m_room_size;
    for (size_t i = 0; i < static_cast<size_t>(rooms_in_row * rooms_in_col * dynamic_objects_ratio); ++i) {
        pos = std::rand() % buffer.size();
        m_context.dynamic_objects.emplace_back(std::move(std::make_unique<Enemy>()));
        m_context.dynamic_objects.back()->set_location(buffer.at(pos));
        buffer.erase(buffer.begin() + pos);
    }

    for (size_t i = 0; i < buffer.size(); ++i) {
        m_context.static_objects.emplace_back(std::move(std::make_unique<Food>()));
        m_context.static_objects.back()->set_location(buffer.at(i));
    }
    buffer.clear();
}

void CommonBuilder::create_state(IStateManager& state_manager,
                                 sf::VideoMode video_mode, const std::string& window_title) {
    m_game_state = std::make_unique<GameState>(state_manager, video_mode, window_title);
}

void CommonBuilder::set_all_to_state() {
    std::vector<std::unique_ptr<Room>> rooms;
    for (auto& row : m_rooms) {
        for (auto& room : row)
            rooms.emplace_back(std::move(room));
    }
    m_game_state->set_maze(Maze(std::move(rooms)));
    m_game_state->set_context(std::move(m_context));
}

std::unique_ptr<GameState> CommonBuilder::get_game() {
    return std::move(m_game_state);
}